//
// File:        RMI.java
// Package:     gov.llnl.babel.backend.rmi
// Copyright:   (c) 2004 The Regents of the University California
// Revision:     @(#) $Id: RMI.java 4673 2005-05-26 23:10:06Z leek2 $
// Description: 
// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the Components Team <components@llnl.gov>
// UCRL-CODE-2002-054
// All rights reserved.
// 
// This file is part of Babel. For more information, see
// http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
// for Our Notice and the LICENSE file for the GNU Lesser General Public
// License.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License (as published by
// the Free Software Foundation) version 2.1 dated February 1999.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
// conditions of the GNU Lesser General Public License for more details.
// 
// You should have recieved a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

package gov.llnl.babel.backend.rmi;

import gov.llnl.babel.BabelConfiguration;
import gov.llnl.babel.backend.CodeConstants;
import gov.llnl.babel.backend.CodeGenerationException;
import gov.llnl.babel.symbols.Type;
import gov.llnl.babel.symbols.Argument;
import gov.llnl.babel.symbols.Interface;
import gov.llnl.babel.backend.c.C;
import gov.llnl.babel.symbols.Class;
import gov.llnl.babel.symbols.Comment;
import gov.llnl.babel.symbols.Enumeration;
import gov.llnl.babel.symbols.Extendable;
import gov.llnl.babel.symbols.Method;
import gov.llnl.babel.symbols.Package;
import gov.llnl.babel.symbols.Symbol;
import gov.llnl.babel.symbols.SymbolID;
import gov.llnl.babel.symbols.Version;
import java.util.Iterator;
import java.util.Collection;

public class RMI { 

  /** generate the method call to pack an argument */
  public static String packArg( String packer_typename, String packer_varname,
                                Argument arg)
  { 
    if(arg.getType().isSymbol()) {
      return packType( packer_typename, packer_varname, arg.getType(), 
                       arg.getFormalName(), arg.getFormalName(), Argument.IN, arg.isCopy());
      //Symbols are special, lets say they're always IN.. HACK
      
    } else {
      return packType( packer_typename, packer_varname, arg.getType(), 
                       arg.getFormalName(), arg.getFormalName(), arg.getMode(),
                       arg.isCopy());
    }
    
  }
   
  /** backwards compatible....*/
 public static String packType( String packer_typename, 
                                   String packer_varname,  Type type, 
                                String varname, String var) {
   String ext = RMI.getMethodExtension( type );
   return packer_typename + "_pack" + ext + "( " + packer_varname + ", \"" 
     + varname + "\", " + var + ", _ex2);";
 }



    /**  generate the method call to pack a type
     * @param packer_typename The string name of the type that implements from 
     *                        sidl.io.Serializer
     * @param packer_varname  The variable (instance) name of the type that 
     *                        implements sidl.io.Serializer
     * @param type The type to be packed
     * @param varname The name of the variable to be packed
     * @param var Same as varname with possible derefencing based on mode
     * @return A string containing the proper 
     */
    public static String packType( String packer_typename, 
                                   String packer_varname,  Type type, 
                                   String varname, String var, int mode, 
                                   boolean isCopy)
  { 
    String ext = RMI.getMethodExtension( type );
    if(type.isSymbol()) {
      if(isCopy) {
        return "ERROR: Serialzation of objects is not yet supported";
      } else {
        String url_string = new String(C.getSymbolName(type.getSymbolID())+
                                       "__getURL(" + varname+")");
        // TODO: This is a really stupid problem, this^^^^^^^^^ should use the skel
        // name.  Simple, figure it out.
        
        return packer_typename + "_packString( " + packer_varname + ", \"" 
          + varname + "\", " + var + ", _ex2);";
       
      }
    }
    switch ( mode ) {
    case Argument.IN:
      return packer_typename + "_pack" + ext + "( " + packer_varname + ", \"" 
        + varname + "\", " + var + ", _ex2);";
    case Argument.OUT:
    case Argument.INOUT:
      return packer_typename + "_pack" + ext + "( " + packer_varname + ", \"" 
        + varname + "\", *" + var + ", _ex2);";
    }

    return "ERROR";
    
  }

    /** generate the method call to unpack an argument
     * @param packer_typename
     * @param packer_varname
     * @param arg
     * @return
     */
    public static String unpackArg( String packer_typename, 
                                    String packer_varname, Argument arg ) 
    { 
        return unpackType( packer_typename, packer_varname, arg.getType(), 
                           arg.getFormalName(), arg.getFormalName() );
    }

    /** generate the method call to unpack a type */
    /**
     * @param packer_typename
     * @param packer_varname
     * @param type
     * @param varname
     * @param var
     * @return
     */
    public static String unpackType( String packer_typename, 
                                     String packer_varname, Type type, 
                                     String varname, String var)
    { 
        String ext = RMI.getMethodExtension( type );
        return packer_typename + "_unpack" + ext + "( " + packer_varname 
          + ", \"" + varname + "\", " + var + ", _ex2);";
    }

    /** generate the proper method extension for packing or unpacking
     * @param t
     * @return
     */
    public static String getMethodExtension( Type t ) { 
        if ( t.isPrimitive() ) { /* fixed size & string */
            String buff = t.getTypeString();  
            /* capitalize first letter */
            return buff.substring(0,1).toUpperCase() + buff.substring(1); 
        } else if ( t.isArray() ) { 
            //FIXME
            return "FIXME";
        } else if ( t.isSymbol() ) { 
            
            return "String";
        } else { 
            return "ERROR";
        }
    }

  /**
   *  generateAnonymousClass takes an interface, and generates the
   *  corrosponding anonymous class.  This anonymous class is only used to
   *  connect an interface remotely.
   *
   */

  public static Class generateAnonymousClass(Interface ifc) throws CodeGenerationException{
    SymbolID ifcID = ifc.getSymbolID();
    String ifcFQN = ifcID.getFullName();
    Version clsVersion = new Version("1.0");
    int index = ifcFQN.lastIndexOf(SymbolID.SCOPE);
    String clsFQN = ifcFQN.substring(0,index+1)+"_"+ifcFQN.substring(index+1);
    SymbolID clsID = new SymbolID(clsFQN, clsVersion);
    Class cls = new Class(clsID, null);
    //Symbol baseClassSymbol = Utilities.lookupSymbol(BabelConfiguration.getBaseClass());
    //cls.setParentClass((Class) baseClassSymbol);
    cls.addParentInterface((Interface)ifc);

    Collection methods = ifc.getMethods(false);
    for(Iterator m = methods.iterator(); m.hasNext(); ) {
      Method method = (Method) m.next();
      cls.addMethod(method);
    }
    return cls;
  }

}
